import xmllib
from RecipeImplementation import RecipeI

from Cookbook import Cookbook
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

addCookbookForm = PageTemplateFile('www/addCookbook.zpt', globals())
addCookbookForm._owner = None

def addCookbookFunction(dispatcher, id, title, REQUEST=None, submit=None) :
    """
    Register a cookbook
    """
    cb = CookbookI(id, title)
    dispatcher.Destination()._setObject(id, cb)

    if REQUEST is not None :
        try : u = dispatcher.DestinationURL()
        except : u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')

    return ''

class CookbookI(Folder) :
    """
    This the class that implements a cookbook.

    It has a list of recipe and can search in this list. It can also add or
    remove a recipe and last but not least it is able to import recipe from an
    xml file.
    """

    __implements__ = Cookbook
    meta_type = 'Cookbook'
    security = ClassSecurityInfo()
    icon = 'www/adimeal_folder.png'

    def __init__(self, id, title) :
        self.id = id
        self.title = title
        self.tot_recipes = 0

    def __str__(self) : return "<Cookbook : %s>" % self.title

    security.declarePublic('findRecipe')
    def findRecipe(self, kind, searchTerm) :
        """
        Goes trought all the recipe in the cookbook asking if they match ther
        searchTerm.
        """
        result = []
        for i in self.objectValues('Recipe') :
            if i.match(kind, searchTerm) : result.append(i)
        return result

    security.declareProtected('Allow Moderation', 'acceptRecipes')
    def acceptRecipes(self, recipe_list) :
        """
        Accept recipes from this list
        """
        if type(recipe_list) == type('') : recipe_list = [recipe_list]
        for i in recipe_list : self._getOb(i).acceptRecipe()

    security.declareProtected('Allow importing', 'importRecipes')
    def importRecipes(self, file) : 
        """
        This is used to import recipes in the cookbook basically with xml
        """
        rec_list = XMLImporter().parse(file)
        for i in rec_list :
            self.tot_recipes += 1
            i.id = 'recipe%s' % self.tot_recipes
            self._setObject('recipe%s' % self.tot_recipes, i)
        
        self._p_changed = 1
        return rec_list
    
    security.declarePublic('incTotRecipe')
    def incTotRecipe(self) :
        " Increase the tot_recipes and returns it "
        self.tot_recipes += 1
        self._p_changed = 1
        return self.tot_recipes

class XMLImporter(xmllib.XMLParser) :
    
    def __init__(self) :
        xmllib.XMLParser.__init__(self, accept_missing_endtag_name=1)
        self.elements = { 
                          'cookbook'   : (self.start_cookbook, None),
                          'recipe'     : (self.start_recipe, self.end_recipe),
                          'ingredient' : (self.start_ingr, None),
                          'time'       : (self.start_time, None),
                          'typist'     : (self.start_typist, None),
                          'source'     : (self.start_source, None),
                          'category'   : (self.start_cat, None),
                          'direction'  : (self.start_direction,
                                          self.end_direction),
                        }

    def parse(self, file) :
        self.cookbook = None
        self.current_recipe = None
        self.howto = ''
        self.isDirection = None
        self.recipe_list = []
        self.num = 0

        line = file.readline()
        while line :
            self.feed(line[:-1])
            line = file.readline()
        file.close()

        self.reset()
        return self.recipe_list

    def syntax_error(self, message) :
        raise 'Syntax Error in importation : %s' % message

    def start_cookbook(self, attr) : pass
        
    def start_recipe(self, attr) :
        self.num += 1
        id = 'recipe%s' % self.num
        self.current_recipe = RecipeI(id, attr['name'])
        self.current_recipe.yield_number = attr['yield']
        
    def end_recipe(self) : 
        self.recipe_list.append(self.current_recipe)
        self.current_recipe = None

    def start_ingr(self, attr) : 
        self.current_recipe.addIngredient(descr=attr['name'],
                                          unite=attr['meas'],
                                          quantite=attr['number'])

    def start_time(self, attr) :
        self.current_recipe.preparation_time = attr.get('ptime')
        self.current_recipe.cooking_time = attr.get('ctime')

    def start_typist(self, attr) :
        self.current_recipe.cook = attr['name']

    def start_source(self, attr) :
        self.current_recipe.source = attr['loc']

    def start_created(self, attr) : pass

    def start_cat(self, attr) :
        self.current_recipe.addCategory(attr['name'])

    def start_direction(self, attr) : self.isDirection = 1

    def end_direction(self) :
        if self.isDirection :
            self.current_recipe.howto.append(self.howto)
            self.isDirection = 0
        else : self.syntax_error('Unmatched </direction>')

    def handle_data(self, data) :
        if self.isDirection :
            if len(data) == 2 :
                if self.howto : self.current_recipe.howto.append(self.howto)
                self.howto = ''
            else :
                self.howto += ' ' + data[2:]
        else : self.syntax_error('Data out of bound')
    
InitializeClass(CookbookI)
