import re
from Recipe import Recipe
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

addRecipeForm = PageTemplateFile('www/addRecipe.zpt', globals())
addRecipeForm._owner = None

def addRecipeFunction(dispatcher, id, name, REQUEST=None, submit=None) :
    """
    Register a recipe
    """
    rec = RecipeI(id, name)
    dispatcher._setObject(id, rec)

    if REQUEST is not None:
        try: u = dispatcher.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

class RecipeI(SimpleItem) :
    """
    This is the class that implement a recipe

    It is in fact just a bunch of variables.
    """

    __implements__ = Recipe
    meta_type = 'Recipe'
    security = ClassSecurityInfo()

    def __init__(self, id, name) :
        self.id = id
        self.title = name
        self.name = name
        self.ingr_list = []
        self.preparation_time = 0
        self.cooking_time = 0
        self.cook = ''
        self.source = ''
        self.yield_number = 0
        self.categories = []
        self.howto = []
        self.moderated = 0

    def __str__(self) : return "<Recipe : %s>" % self.name
    
    security.declareProtected('Change Recipe', 'addIngredient')
    def addIngredient(self, quantite, unite, descr) :
        """ Add an ingredient to the indredient list """
        self.ingr_list.append({'quant' : quantite, 'unit'  : unite, 
                               'what'  : descr})
        self.hasChanged()

    security.declareProtected('Change Recipe', 'addCategory')
    def addCategory(self, category) :
        """ add a category to the category list """
        self.categories.append(category)
        self.hasChanged()

    security.declareProtected('Change Recipe', 'setPrepTime')
    def setPrepTime(self, time) :
        """ Set the preparation time """
        self.preparation_time = time
        self.hasChanged()

    security.declareProtected('Change Recipe', 'setCookTime')
    def setCookTime(self, time) :
        """ Set the cooking time """
        self.cooking_time = time
        self.hasChanged()

    security.declareProtected('Change Recipe', 'setCook')
    def setCook(self, cook) :
        """ Who's the cook ? """
        self.cook = cook
        self.hasChanged()

    security.declareProtected('Change Recipe', 'setSource')
    def setSource(self, source) :
        """ Where does this recipe come from """
        self.source = source
        self.hasChanged()

    security.declareProtected('Change Recipe', 'appendDirectionPar')
    def appendDirectionPar(self, dir) :
        " Append a paragraph to the howto "
        self.howto.append(dir)
        self.hasChanged()

    security.declareProtected('Change Recipe', 'setYield')
    def setYield(self, yieldn) :
        """ For how many people ? """
        self.yield_number = yieldn
        self.hasChanged()

    security.declareProtected('Moderate this recipe', 'acceptRecipe')
    def acceptRecipe(self) :
        """ Accept this recipe. It can now be published """
        self.moderated = 1
        self.hasChanged()

    security.declarePublic('match')
    def match(self, kind, term) :
        """ Does this recipe match this search ? """
        if kind == 'name' : 
            bool = re.search(term, self.name, re.I) # Case insensitive
        elif kind == 'category' : 
            def up(s) : return s.upper()
            bool = term.upper() in map(up, self.categories)
        elif kind == 'ingredient' :
            bool = None
            for i in self.ingr_list :
                if re.search(term, i['what'], re.I) :
                    bool = 1
                    break
        elif kind == 'nonapproved' : bool = not self.moderated
        else : raise 'ValueError', kind

        return bool or (not self.moderated and kind == 'nonapproved')
    
    def hasChanged(self) :
        """
        Make Zope remember the changes
        """
        self._p_changed = 1

InitializeClass(RecipeI)
